from django.contrib.auth import get_user_model


def create_user(activation_token, **kwargs):
    user = get_user_model().objects.create_user(
        email=activation_token.email,
        username=kwargs.pop("username", activation_token.email),
        accepts_terms=activation_token.accepts_terms,
        terms_version_accepted=activation_token.terms_version_accepted,
        **kwargs,
    )
    return user



import django_rtk.mutations as mutations
import graphene
from django_rtk.utils import get_validator


class ChangePassword(mutations.ChangePassword):
    class Arguments:
        email = graphene.String()
        password = graphene.String()
        new_password = graphene.String()

    @classmethod
    def validate_args(cls, errors, email, new_password, **kwargs):
        get_validator().validate_email(errors, email)
        get_validator().validate_password(errors, new_password)



import django_rtk.mutations as mutations
import graphene
from django_rtk.utils import get_setting_or, get_validator


class ActivateAccount(mutations.ActivateAccount):
    class Arguments:
        activation_token = graphene.String()
        password = graphene.String()
        if get_setting_or(False, "REQUIRE_USERNAME"):
            username = graphene.String()

    @classmethod
    def validate_args(cls, errors, password, **kwargs):
        get_validator().validate_password(errors, password)
        if get_setting_or(False, "REQUIRE_USERNAME"):
            get_validator().validate_username(errors, kwargs["username"])



import uuid

from django.contrib.auth import authenticate, get_user_model
from django_rtk.utils import count_errors, get_errors

from django_rtk_green.models import ActivationToken, PasswordResetToken
from django_rtk_green.utils import create_user


class Backend:
    def register_account(
        self, errors, email, accepts_terms, terms_version_accepted, **kwargs
    ):
        result = dict(activation_token="")
        if count_errors(errors):
            return result

        if get_user_model().objects.filter(email=email).exists():
            get_errors(errors, "email").append("ALREADY_TAKEN")
            return result

        activation_token, _ = ActivationToken.objects.get_or_create(
            email=email,
            defaults=dict(
                accepts_terms=accepts_terms,
                terms_version_accepted=terms_version_accepted,
            ),
        )
        result["activation_token"] = activation_token.token.hex
        return result

    def activate_account(self, errors, activation_token, **kwargs):
        result = dict(user=None)
        if count_errors(errors):
            return result

        activation_token = ActivationToken.objects.filter(
            token=uuid.UUID(activation_token)
        ).first()
        if activation_token:
            user = result["user"] = create_user(activation_token, **kwargs)
            activation_token.delete()
        else:
            user = None

        if not user:
            get_errors(errors, "activation_token").append("NOT_FOUND")

        return result

    def request_password_reset(self, errors, email, **kwargs):
        result = dict(password_reset_token="")
        if count_errors(errors):
            return result

        if (
            not get_user_model().objects.filter(email=email).exists()
            and not ActivationToken.objects.filter(email=email).exists()
        ):
            get_errors(errors, "email").append("ACCOUNT_UNKNOWN")
            return result

        password_reset_token, _ = PasswordResetToken.objects.get_or_create(email=email)
        result["password_reset_token"] = password_reset_token.token.hex

        return result

    def reset_password(self, errors, password_reset_token, password, **kwargs):
        result = dict(user=None)
        if count_errors(errors):
            return result

        password_reset_token = PasswordResetToken.objects.filter(
            token=uuid.UUID(password_reset_token)
        ).first()

        if not password_reset_token:
            get_errors(errors, "password_reset_token").append("NOT_FOUND")
            return result

        user = result["user"] = (
            get_user_model().objects.filter(email=password_reset_token.email).first()
        )

        if user:
            user.set_password(password)
            user.save()
            password_reset_token.delete()
        else:
            get_errors(errors, "password_reset_token").append("ACCOUNT_UNKNOWN")

        return result

    def change_password(self, errors, email, password, new_password, **kwargs):
        result = dict(user=None)
        if count_errors(errors):
            return result

        user = get_user_model().objects.filter(email=email).first()
        if not user:
            get_errors(errors, "email").append("ACCOUNT_UNKNOWN")
            return result

        username = getattr(user, get_user_model().USERNAME_FIELD)
        is_authenticated = authenticate(username=username, password=password)
        if not is_authenticated:
            get_errors(errors, "password").append("INVALID_CREDENTIALS")
            return result

        user.set_password(new_password)
        user.save()
        return result



from django_rtk.utils import get_setting_or


def extract_token(result, token_name):
    token = result.get(token_name, "")
    if not get_setting_or(False, "DANGEROUSLY_EXPOSE_TOKENS"):
        token = ""
    return token



import django_rtk.mutations as mutations
import graphene
from django_rtk.utils import get_backend, get_errors, get_setting_or, get_validator
from django_rtk_green.mutations.utils import extract_token


class RegisterAccount(mutations.RegisterAccount):
    class Arguments:
        email = graphene.String()
        accepts_terms = graphene.Boolean()
        terms_version_accepted = graphene.String(
            required=False, default_value=get_setting_or("1.0.0", "TERMS_VERSION")
        )

    activation_token = graphene.String()

    @classmethod
    def validate_args(cls, errors, email, **kwargs):
        get_validator().validate_email(errors, email)

    @classmethod
    def run(cls, errors, **kwargs):
        result = get_backend().register_account(errors, **kwargs)

        hide_account_existence = get_setting_or(True, "HIDE_ACCOUNT_EXISTENCE")
        if hide_account_existence and "ALREADY_TAKEN" in get_errors(errors, "email"):
            get_errors(errors, "email").remove("ALREADY_TAKEN")

        return result

    @classmethod
    def send_email(cls, result, **kwargs):
        mutations.send_activation_email(result, **kwargs)

    @classmethod
    def get_output_values(cls, result):
        return {"activation_token": extract_token(result, "activation_token")}



import graphene
from django_rtk.mutations import ObtainJSONWebToken

from .activateaccount import ActivateAccount
from .changepassword import ChangePassword
from .registeraccount import RegisterAccount
from .requestpasswordreset import RequestPasswordReset
from .resetpassword import ResetPassword


class Mutation(graphene.ObjectType):
    register_account = RegisterAccount.Field()
    activate_account = ActivateAccount.Field()
    request_password_reset = RequestPasswordReset.Field()
    reset_password = ResetPassword.Field()
    change_password = ChangePassword.Field()
    token_auth = ObtainJSONWebToken.Field()
